Attribute VB_Name = "Common"
Option Explicit
' --- GPL ---
'
' Copyright (C) 2000-2001 SAP AG
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' --- GPL ---
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
'  Module:     Common
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'--------------------------------------------------------
'module name
'--------------------------------------------------------
Const MODULE_NAME = "Common"

Public Const TABLE_DOMAIN_SYNONYMS = "DOMAIN.SYNONYMS"


Public Const COL_DOMAIN_TABLEOWNER = "TABLEOWNER"
Public Const COL_DOMAIN_TABLENAME = "TABLENAME"
Public Const TAB_DOMAIN_SYNONYMS = "DOMAIN.SYNONYMS"
Public Const DBA_COL_DOMAIN_OWNER = "OWNER"
Public Const COL_DOMAIN_SYNONYMNAME = "SYNONYMNAME"

'Global Const SQL_DATA_AT_EXEC = -2
'Global Const SQL_NEED_DATA = 99
'Global Const SQL_NO_DATA_FOUND = 100
'Global Const SQL_SUCCESS = 0
'Global Const SQL_SUCCESS_WITH_INFO = 1
'Global Const SQLSTATE_SUCCESS = "00000"
'Global Const SQLSTATE_DATA_TRUNCATED = "01004"

Public m_oComFunc As New SQLStudioUtil.Functions

Public Sub internalError(ByVal CurrentError As ErrObject, ModulName As String, FunctionName As String)
    MsgBox "Module:" + Chr$(9) + ModulName + Chr$(13) + Chr$(10) + "Function:" + Chr$(9) + FunctionName + Chr$(13) + Chr$(10) + "Error No:" + Chr$(9) + Trim$(Str$(CurrentError.Number)) + Chr$(13) + Chr$(10) + "Error:" + Chr$(9) + CurrentError.Description, vbExclamation, "Unexpected Error:"
End Sub

'=========================================================================================
'    function:      commonGetParameters
'    description:   replaces any occurrence of "[*]" (where '*' denotes arbitrary text)
'                   in the argument string with text entered into a textbox in a modal popup window.
'                   replacement is not performed inside SAPDB SQL string literals (e.g. "'lala'"),
'                   and special identifiers ( e.g. ""pipo"").
'                   one ore more "[" may occur between the enclosing "[" and "]".
'    arguments:     the_statement, typically an SQL statement.
'
'    return value:  false if an error occurs, true otherwise
'                   ( the only error that is detected here is a missing "]" )
'
'=========================================================================================
Public Function commonGetParameters(the_statement As String) As Boolean
    On Error GoTo errCommonGetParameters
    
    Const FUNCTION_NAME = "commonGetParameters"
    
    Const sl_delim = "'"
    Dim si_delim As String
    si_delim = Chr$(34)
    Const par_left_delim = "["
    Const par_right_delim = "]"
    
    commonGetParameters = True
    
     
    Dim the_len                 As Long
    Dim the_modified_statement  As String
    Dim i As Integer
    
    Dim parameter_offsets       As New Collection
    Dim parameter_names         As New Collection
    Dim parameter_values        As Collection
    Dim accumulated_offset      As Integer
    Dim parcnt                  As Integer
    
    Dim read_char   As String
    
    Dim lookahead   As String
    Dim par_name    As String
    Dim par_value   As String
     
    Dim in_sl       As Boolean
    Dim in_si       As Boolean
    Dim in_par      As Boolean
    
    Dim in_escaped_delim As Boolean
    
    the_len = Len(the_statement)
    the_modified_statement = ""
    
    in_sl = False
    in_si = False
    in_par = False
    accumulated_offset = 0
    
    in_escaped_delim = False
    
    For i = 1 To the_len
        read_char = Mid$(the_statement, i, 1)
        If i = the_len Then
            lookahead = 0
        Else
            lookahead = Mid$(the_statement, i + 1, 1)
        End If
        
        ' state : are we in a special identifier ?
        If (in_sl) And (Not in_si) And (Not in_par) _
        And read_char = sl_delim Then
            If in_escaped_delim Then
                in_escaped_delim = False
            Else
                If lookahead <> sl_delim Then
                    If Not in_escaped_delim Then in_sl = False
                Else
                    in_escaped_delim = True
                End If
            End If
        Else
            If (Not in_sl) And (Not in_si) And (Not in_par) _
                And read_char = sl_delim Then in_sl = True
        End If
        
        ' state : are we in a string literal ?
        If (Not in_sl) And (in_si) And (Not in_par) _
        And read_char = si_delim Then
            If in_escaped_delim Then
                in_escaped_delim = False
            Else
                If lookahead <> si_delim Then
                    If Not in_escaped_delim Then in_si = False
                Else
                    in_escaped_delim = True
                End If
            End If
        Else
            If (Not in_sl) And (Not in_si) And (Not in_par) _
                And read_char = si_delim Then in_si = True
        End If
        
        ' state : are we between square brackets outside strings and special identifiers ?
        If (Not in_sl) And (Not in_si) And (Not in_par) _
        And read_char = par_left_delim Then
                in_par = True
                par_name = ""
                read_char = ""
        End If
        
        If (Not in_par) Then the_modified_statement = the_modified_statement + read_char
            
        If (Not in_sl) And (Not in_si) And (in_par) Then
            If read_char = par_right_delim Then
                in_par = False
                parameter_offsets.add (Len(the_modified_statement))
                parameter_names.add (par_name)
            Else
                par_name = par_name + read_char
            End If
        End If
        
        
    Next i
    
    Set parameter_values = get_replacements(parameter_names, the_statement)
    
    ' paste the entered parameter values into the statement :
    If parameter_values.count > 0 Then
        For parcnt = 1 To parameter_values.count
            the_modified_statement = left(the_modified_statement, parameter_offsets(parcnt) + accumulated_offset) + _
            parameter_values(parcnt) + _
            Right(the_modified_statement, Len(the_modified_statement) - parameter_offsets(parcnt) - accumulated_offset)
            accumulated_offset = accumulated_offset + Len(parameter_values(parcnt))
        Next parcnt
        the_statement = the_modified_statement
    End If
    
    Exit Function
    
errCommonGetParameters:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function

'=========================================================================================
'    function:      get_replacements
'    description:   opens a modal dialog box with text entry fields.
'    arguments:     par_names, collection of captions to the text fields.
'                   the_statement, for the caption of the dialog box.
'    return value:  what has been entered, in a collection
'
'=========================================================================================
Private Function get_replacements(par_names As Collection, the_statement As String) As Collection
    On Error GoTo errget_replacements
    
    Const FUNCTION_NAME = "get_replacements"
    
    
    Dim name As Variant
    Dim lParamCount As Integer
    lParamCount = 0
    
    Dim oFormParam         As frmGetParameters

    Set get_replacements = New Collection
    
    If par_names.count > 0 Then
        Set oFormParam = New frmGetParameters
        For Each name In par_names
            lParamCount = lParamCount + 1
            Load oFormParam.lblParameter(lParamCount)
            Load oFormParam.txtParameter(lParamCount)
            oFormParam.lblParameter(lParamCount).Caption = name
        
            'position new objects
            oFormParam.lblParameter(lParamCount).top = _
                            (lParamCount - 1) * _
                            (oFormParam.lblParameter(lParamCount).height + 60 + _
                             oFormParam.txtParameter(lParamCount).height + 60)
            oFormParam.txtParameter(lParamCount).top = oFormParam.lblParameter(lParamCount).top + (oFormParam.lblParameter(lParamCount).height + 60)
        
            If oFormParam.vsbColumns.Visible Then
        
                oFormParam.txtParameter(lParamCount).width = oFormParam.ScaleWidth - oFormParam.txtParameter(lParamCount).left - oFormParam.vsbColumns.width - 120
            Else
                oFormParam.txtParameter(lParamCount).width = oFormParam.ScaleWidth - oFormParam.txtParameter(lParamCount).left - 120
            End If
            'make visible
            oFormParam.lblParameter(lParamCount).Visible = True
            oFormParam.txtParameter(lParamCount).Visible = True
        
    
        Next name
        
        oFormParam.Caption = oFormParam.Caption + " for " + left(the_statement, 10)
        If Len(the_statement) > 9 Then _
           oFormParam.Caption = oFormParam.Caption + "..."
        
        oFormParam.Show vbModal
        
        If oFormParam.OK = False Then
            Set oFormParam = Nothing
        Else
            Set get_replacements = oFormParam.Parameters
        End If
    End If
    Exit Function
    
errget_replacements:
    internalError Err, MODULE_NAME, FUNCTION_NAME
End Function
